# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
import cgi
import thread

# Import from itools
from itools import uri
from itools.resources import zope2
from itools.handlers import get_handler
from itools.catalog import IIndex, IO
from itools.zope import get_context

# Import from ikaaro
from Products import ikaaro
from Products.ikaaro.Metadata import Metadata
from Products.ikaaro.Root import Root
from Products.ikaaro.Handler import Handler



def manage_ikaaro(self, path='/', update='0'):
    context = get_context()
    response = context.response

    # Load the root handler
    metadata_object = getattr(self, '.metadata')
    root_path = metadata_object.getPhysicalPath()[:-1]
    root_resource = zope2.get_resource('/'.join(root_path))
    root = ikaaro.get_root(root_resource)

    # Get the handler
    path = uri.Path(path)
    handler = root.get_handler(path)

    # Update?
    if update == '1':
        handler.update()
        response.redirect('manage_ikaaro')
        return

    # Build the namespace
    namespace = {}
    namespace['thread_id'] = thread.get_ident()
    # The resource
    dict = handler.resource.__dict__.items()
    dict.sort()
    dict = [ {'name': str(name), 'value': cgi.escape(str(value))}
             for name, value in dict ]
    namespace['resource'] = {'repr': cgi.escape(repr(handler.resource)),
                             'dict': dict}
    # The handler
    namespace['handler'] = {
        'repr': cgi.escape(repr(handler)),
        'resource': cgi.escape(repr(handler.resource)),
        'timestamp': str(handler.timestamp),
        'abspath': handler.get_abspath() or '/',
        'dict': dict,
        'is_obsolete': False
        }
    # State
    dict = []
    for name, value in handler.state.__dict__.items():
        if name == 'parent':
            value = '<a href="manage_ikaaro?path=%s">%s</a>' \
                    % (value.abspath, cgi.escape(repr(value)))
        elif name == 'cache':
            value = [ '<a href="manage_ikaaro?path=%s">%s</a>: %s'
                      % (path.resolve2(x), x, cgi.escape(repr(y)))
                      for x, y in value.items() ]
            value.sort()
            value = '<br/>'.join(value)
        else:
            value = cgi.escape(str(value))
        dict.append({'name': name, 'value': value})
    dict.sort(lambda x, y: cmp(x['name'], y['name']))
    namespace['handler']['state'] = {'value': cgi.escape(repr(handler.state)),
                                     'dict': dict}
    # Parent, name, real handler
    if handler.parent is None:
        namespace['handler']['parent'] = {'href': '', 'value': ''}
    else:
        href = 'manage_ikaaro?path=%s' % handler.parent.abspath
        value = cgi.escape(repr(handler.parent))
        namespace['handler']['parent'] = {'href': href, 'value': value}
    namespace['handler']['name'] = handler.name
    real_handler = cgi.escape(repr(handler.real_handler))
    namespace['handler']['real_handler'] = real_handler

    if isinstance(handler, Handler):
        class_version = handler.class_version
        version = handler.get_property('version')
        namespace['handler']['class_version'] = class_version
        namespace['handler']['version'] = version
        namespace['handler']['is_obsolete'] = class_version > version
    # Catalog
    namespace['is_iindex'] = isinstance(handler, IIndex.IIndex)
    if namespace['is_iindex']:
        # Tree Header
        tree_handler = handler.state.tree_handler
        namespace['handler']['tree'] = {}
        namespace['handler']['tree']['version'] = tree_handler.state.version
        number_of_slots = tree_handler.state.number_of_slots
        namespace['handler']['tree']['number_of_slots'] = number_of_slots
        first_slot = tree_handler.state.first_slot
        if first_slot is None:
            namespace['handler']['tree']['first_slot'] = ''
        else:
            namespace['handler']['tree']['first_slot'] = first_slot
        first_empty = tree_handler.state.first_empty
        if first_empty is None:
            namespace['handler']['tree']['first_empty'] = ''
        else:
            namespace['handler']['tree']['first_empty'] = first_empty
        # Tree Slots
        slots = []
        tree_rsrc = handler.state.tree_handler.resource
        for tree_slot_n in range(number_of_slots):
            # Load the slot
            tree_slot = 16 + tree_slot_n * 16
            tree_slot_data = tree_rsrc[tree_slot:tree_slot+16]
            c = IO.decode_character(tree_slot_data[0:4])
            doc_slot_n = IO.decode_link(tree_slot_data[4:8])
            child_n = IO.decode_link(tree_slot_data[8:12])
            sibling_n = IO.decode_link(tree_slot_data[12:16])
            #
            if doc_slot_n is not None:
                doc_slot_n = {'name': '%03d' % doc_slot_n,
                              'href': '#%03d' % doc_slot_n}
            if child_n is not None:
                child_n = {'name': '%03d' % child_n,
                           'href': '#%03d' % child_n}
            if sibling_n is not None:
                sibling_n = {'name': '%03d' % sibling_n,
                             'href': '#%03d' % sibling_n}
            slots.append({'index': '%03d' % tree_slot_n,
                          'char': c, 'doc_slot_n': doc_slot_n,
                          'child_n': child_n, 'sibling_n': sibling_n})
        namespace['handler']['tree']['slots'] = slots
        # Docs Header
        docs_handler = handler.state.docs_handler
        docs_rsrc = handler.state.docs_handler.resource
        namespace['handler']['docs'] = {}
        namespace['handler']['docs']['version'] = docs_handler.state.version
        number_of_slots = docs_handler.state.number_of_slots
        namespace['handler']['docs']['number_of_slots'] = number_of_slots
        first_empty = docs_handler.state.first_empty
        if first_empty is None:
            namespace['handler']['docs']['first_empty'] = ''
        else:
            namespace['handler']['docs']['first_empty'] = first_empty
        # Free blocks
        free_blocks = set()
        free_block_n = docs_handler.state.first_empty
        while free_block_n is not None:
            free_blocks.add(free_block_n)
            free_block = 12 + free_block_n * 4
            free_block_n = IO.decode_link(docs_rsrc[free_block+4:free_block+8])
        # Docs Slots
        slots = []
        block_index = 0
        slot_index = 0
        while slot_index < number_of_slots:
            doc_slot = 12 + slot_index * 4
            if slot_index in free_blocks:
                header = docs_rsrc[doc_slot:doc_slot+8]
                size = IO.decode_uint32(header[0:4])
                next = IO.decode_link(header[4:8])
                slots.append({'block_index': '%03d' % block_index,
                              'slot_index': '%03d' % slot_index,
                              'doc_number': None,
                              'frequency': None,
                              'next': next,
                              'positions': None})
                slot_index = slot_index + size
            else:
                # Load header
                header = docs_rsrc[doc_slot:doc_slot+12]
                doc_number = IO.decode_uint32(header[0:4])
                frequency = IO.decode_uint32(header[4:8])
                docs_slot_n = IO.decode_link(header[8:12])
                # Load positions
                positions = docs_rsrc[doc_slot+12:doc_slot+12+(frequency*4)]
                positions = [ IO.decode_uint32(positions[x*4:x*4+4])
                              for x in range(frequency) ]
                positions = ', '.join([ str(x) for x in positions ])
                if docs_slot_n is not None:
                    docs_slot_n = {'name': '%03d' % docs_slot_n,
                                  'href': ('#%s' % docs_slot_n)}
                slots.append({'block_index': '%03d' % block_index,
                              'slot_index': '%03d' % slot_index,
                              'doc_number': doc_number,
                              'frequency': frequency,
                              'next': docs_slot_n,
                              'positions': positions})
                slot_index = slot_index + frequency + 3
            block_index += 1
        namespace['handler']['docs']['slots'] = slots

    # Load the template
    response.set_header('Content-Type', 'text/html; charset=UTF-8')
    path = '%s/Extensions/manage_ikaaro.xhtml' % ikaaro.__path__[0]
    template = get_handler(path)
    return template.stl(namespace)
